# -*- coding: utf-8 -*-
from django.shortcuts import *
from django.core.files import File
from django.template import RequestContext
from django.http import *
from django import forms
from django.core.urlresolvers import reverse
from django.template import Context, loader
from django.contrib.auth.decorators import login_required


import simplejson
from jsonrpc import JSONRPCService, jsonremote

from models import *

@login_required()
def main_content(request):
	return main(request, only_content=True)
	
@login_required()
def main(request, only_content = False):
	"""Main screen, intended to give the user a sense of what is going
	on in one glance, and the ability to take the most common actions. """
	try:
		game = request.user.game
		game.check_way()
		game = Game.objects.get(id=request.user.game.id)
	except Game.DoesNotExist:
		game = create_game(request.user)		

	# A clumsy way to reload the game object... needed to ensure te way_progress
	# is current...
	
	try:
		game.way_progress
		travelling = True
	except WayProgress.DoesNotExist:
		travelling = False
	
	if travelling:
		delta = game.way_progress.end_time - datetime.now()
		d =dict(travelling= True, destination = game.way_progress.market_b, \
			arrival_time = game.way_progress.end_time, microseconds=delta.seconds*1000
		)
	else:	
		stock  =[]
		inventory = {}
	
		for bunch in game.feather_bunches.all():
			inventory[bunch.species.id] = bunch.count			
		for feather_stock in game.location.feather_stocks.all():
			try:
				players_inventory = inventory[feather_stock.species.id]
			except KeyError:
				players_inventory = 0
			row = {"species":feather_stock.species, "count":feather_stock.count, "price":feather_stock.price_per_feather, 'inventory': players_inventory, "id":feather_stock.species.id}
			stock.append(row)		
		# Sort markets out
		markets ={}
		
		market_ids = []
		maps_on_sale = []
		
		travel_to =[]
		for market in game.maps.all():
			markets[market.id] = market
			if game.location!=market:
				market.distance = int(market.distance_to(game.location))
				travel_to.append(market)
		
		for market in Market.objects.all():
			if not markets.has_key(market.id):
				maps_on_sale.append(market)
		
		
			
		d = dict(travelling = False, game=game, stock=stock, travel_to=travel_to, maps_on_sale=maps_on_sale)
		
	if only_content:
		tname = "game/main_content.html"
	else:
		tname = "game/main.html"
	return render_to_response(tname, d, context_instance=RequestContext(request))


def buy (request):
	return transaction(request, buy=True)
def sell (request):
	return transaction(request, buy=False)
def transaction(request, buy = True):
	"""This view is only called by the javascript code. It always returns a jsonified dictionary. The "html" parameter in there is the content of the dialog box. The other key/value pairs is for the javascript code itself.
	
	In full gameplay, the price should change at least every five minutes, and
	other players might have bought all the available feathers anyway, so the
	offer must be as up-to-date as possible.
	
	The way I chose to do this might look clumsy, but it works quite nicely.
	
	Note that transaction is called by both for purchases and sales. 
	"""
	game = request.user.game
	id = request.GET["id"]	
	species = BirdSpecies.objects.get(id=id)	
	feather_stock = game.location.feather_stocks.filter(species=species)[0]
	try:
		feather_bunch = game.feather_bunches.get(species=species)
	except FeatherBunch.DoesNotExist:
		feather_bunch = FeatherBunch(species=species,count=0, game=game)
		# BTW: you might have noticed feather_bunch isn't saved just yet.
		# It only gets created when the purchase is made, which is not always the
		# case when this code is called.
		
	if buy:
		max_number = min(feather_stock.count, game.coins / feather_stock.price_per_feather)
	else:
		max_number = feather_bunch.count
		
	# On "second pass", the "confirm" parameter is true, and we try to execute the trade if possible
	if request.GET.has_key("confirm"):
		# Test if offer is still valid.
		# If price has changed, or not enough feathers are available,
		#  the trade must be broken off
		
		number = int(request.GET["number"])
		price = int(request.GET["price"])
		current_price = feather_stock.price_per_feather

		if (number > max_number) or \
			(price!=current_price):
			# Return Failure
			d = dict(success=0)
			return HttpResponse(simplejson.dumps(d))
		# Otherwise execute transaction
		if buy:
			feather_stock.count -= number
			feather_bunch.count += number
			game.coins -= feather_stock.price_per_feather * number
		else:
			feather_stock.count += number
			feather_bunch.count -= number
			game.coins += feather_stock.price_per_feather * number
			
		# Don't forget to save the changes
		feather_stock.save()
		feather_bunch.save()
		game.save()		
		
		# Return success. The javascript side will reload
		# the content div afterwards.
		
		d = dict(success=1)
		return HttpResponse(simplejson.dumps(d))
		
	# On first pass, the "buy" parameter is false. Render the dialog box and
	# return info dictionary
	
	c = Context(dict(species = species, max_number=max_number, \
	 price = feather_stock.price_per_feather, buy=buy))
	
	html = loader.get_template("game/buy.html").render(c)
	
	d = dict(id=id, html=html, max_number=max_number, \
			price= feather_stock.price_per_feather)
	
	return HttpResponse(simplejson.dumps(d))

def buy_map(request):
	market_id = request.GET["id"]
	
	game = request.user.game
	market = Market.objects.get(id=market_id)
	if game.coins>= market.map_price:
		
		game.maps.add(market)
		game.coins -= market.map_price;
		game.save()
		
		d = dict(success=1)
	else:
		d = dict(success=0)	
	return HttpResponse(simplejson.dumps(d))


def travel_to(request):
	market_id = int(request.GET["id"])
	market = Market.objects.get(id=market_id)
	game = request.user.game
	
	if game.maps.filter(id=market_id).count()>0:
	
		
		game.travel_to(market)
		#game.location = None
		#game.location = market
		game.save()
		pass
	else:
		pass
	d = dict(success=1)
	return HttpResponse(simplejson.dumps(d))
def net_worth(request):
	highscore = User.objects.all().exclude(game=None).order_by("-game__net_worth")

	d = dict(success=1, highscore= highscore)
	return render_to_response("game/net_worth.html", d, context_instance=RequestContext(request))

